<?php

namespace Ecommerce\Controllers;

use Ecommerce\Util\Utilities;
use Ecommerce\Services\WebServiceAFService;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\DTO;

class WebServiceAF
{
    protected $utilities;
    protected $service;
    protected $newCodeSC;
    
    private $readAFs = array();
    
    public function __construct(Utilities $utilities, WebServiceAFService $service, NewCode $newCodeSC)
    {
        $this->utilities = $utilities;
        $this->service = $service;
        $this->newCodeSC = $newCodeSC;
	}
	
	public function getAvailableAFs(DTO\Request $request, DTO\Response $response)
	{
		try {
			$this->service->beginTransaction();
			
			$req = $request->getParameters();
		    
		    $mainSupplier = $this->checkAccess($req);
		    
		    $availableAFs = $this->getFilteredAvailableAFs($mainSupplier, $req);
		    
		    $afsJson = $this->buildFormattedJson($availableAFs);
		    
		    $this->processReadAFs();
		    
		    $afsFile = new DTO\Response\File("", false, null, "application/json", $afsJson);
		    $response->setFileToDownload($afsFile);
			
		    $this->service->commit();
		} catch(\Exception $e) {
			$this->service->rollback();
			$error = new DTO\Response\Error($e->getMessage(), $e->getCode());
			$response->setError($error);
		}
	}
	
	private function checkAccess($req)
	{
		$mainSupplier = !empty($req['cnpjPrincipal']) ? $this->utilities->removeSpecialCharacters($req['cnpjPrincipal']) : false;
		$accessKey = !empty($req['chaveAcesso']) ? $req['chaveAcesso'] : false;
		
	    if ($mainSupplier && $accessKey) {
	        $mainSupplier = $this->service->getMainSupplier($mainSupplier, $accessKey);
	        
	        if (!$mainSupplier) {
	            throw new \Exception("Acesso inválido.");
	        }
	        
	        return $mainSupplier;
	    } else {
	        throw new \Exception("Acesso inválido.");
	    }
	}
	
	private function getFilteredAvailableAFs($mainSupplier, $req)
	{
		$state = $this->processState($req);
		$afCode = $this->processAfCode($req);
		$status = $this->processStatus($req);
		list($initialDate, $finalDate, $allDate) = $this->processDate($req);
		
		$availableAFs = $this->service->getAvailableAFs($mainSupplier['CDFORNECED'], $initialDate, $finalDate, $allDate, $afCode, $state, $status);
		
		return $availableAFs;
	}
	
	private function processState($req)
	{
		if (!empty($req['uf'])) {
			if (is_array($req['uf'])) {
				return implode("_", $req['uf']);
			} else {
				throw new \Exception("Caso informada, a propriedade 'uf' deve ser uma lista.");
			}
		} else {
			return "ALL";
		}
	}
	
	private function processAfCode($req)
	{
		if (!empty($req['numeroAF'])) {
			if (is_array($req['numeroAF'])) {
				$req['numeroAF'] = array_map(function($value) {
					if (strlen($value) > 10) {
						return substr($value, -10);
					} else {
						return str_pad($value, 10, '0', STR_PAD_LEFT);
					}
				}, $req['numeroAF']);
				
				return implode("_", $req['numeroAF']);
			} else {
				throw new \Exception("Caso informada, a propriedade 'numeroAF' deve ser uma lista.");
			}
		} else {
			return "ALL";
		}
	}
	
	private function processStatus($req)
	{
		if (!empty($req['status'])) {
			$status = strtoupper($req['status']);
			
			if ($status == "LIDO" || $status == "DISPONIVEL") {
				return $status;
			} else {
				throw new \Exception("Caso informada, a propriedade 'status' deve ser 'LIDO' ou 'DISPONIVEL'.");
			}
		} else {
			return "ALL";
		}
	}
	
	private function processDate($req)
	{
		if (!empty($req['dataEntrega'])) {
			$deliveryDate = $req['dataEntrega'];
			
			if (!empty($deliveryDate['inicial']) && !empty($deliveryDate['final'])) {
				$initialDate = \DateTime::createFromFormat( "d/m/Y", $deliveryDate['inicial']);
				$finalDate = \DateTime::createFromFormat( "d/m/Y", $deliveryDate['final']);
				
				if ($initialDate && $finalDate) {
					$initialDate = $initialDate->format('d/m/Y');
					$finalDate = $finalDate->format('d/m/Y');
					
					return array($initialDate, $finalDate, null);
				} else {
					throw new \Exception("Caso informada, o formato das datas devem ser 'DD/MM/YYYY'.");
				}
			} else if (empty($deliveryDate['inicial']) xor empty($deliveryDate['final'])) {
				throw new \Exception("Caso informado, o período deve estar completo.");
			} else {
				return array(null, null, "ALL");
			}
		} else {
			return array(null, null, "ALL");
		}
	}
	
	private function buildFormattedJson($availableAFs)
	{
		if (count($availableAFs)) {
			$afList = array();
			$productList = array();
			
			$af = $this->buildAF($availableAFs[0]);
			$prevIdAF = $availableAFs[0]['NUMERO'];
			
			foreach ($availableAFs as $availableAF) {
				$product = $this->buildProduct($availableAF);
				
				if ($availableAF['NUMERO'] <> $prevIdAF) {
					$af['produtos'] = $productList;
					array_push($afList, $af);
					
					$af = $this->buildAF($availableAF);
					$productList = array();
					
					$prevIdAF = $availableAF['NUMERO'];
				}
				
				array_push($productList, $product);
			}
			
			$af['produtos'] = $productList;
			array_push($afList, $af);
			
			return json_encode($afList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
		} else {
			throw new \Exception("Não foram encontradas Autorizações de Fornecimento.");
		}
	}
	
	private function buildAF($availableAF)
	{
		$deliveryInfo = $this->buildDeliveryInfo($availableAF);
		$collectionInfo = $this->buildCollectionInfo($availableAF);
		$billingInfo = $this->buildBillingInfo($availableAF);
		$typeAF = $this->getTypeAF($availableAF['TIPO']);
		$situationAF = $this->getSituationAF($availableAF['SITUACAOBAIXA']);
		$this->buildLogReadAFs($availableAF);
		
		$af = array(
			"cnpj" => $availableAF['CNPJ'],
			"nome" => $availableAF['NOME'],
			"razao" => $availableAF['RAZAO'],
			"numero" => $availableAF['NUMERO'],
			"dataCriacao" => substr($availableAF['DATACRIACAO'], 0, 10),
			"dataEntrega" => substr($availableAF['DATAENTREGA'], 0, 10),
			"unidadeDestino" => $availableAF['UNIDADEDESTINO'],
			"nomeUnidadeDestino" => $availableAF['NOMEUNIDADEDESTINO'],
			"uf" => $availableAF['UF'],
			"tipo" => $typeAF,
			"situacaoBaixa" => $situationAF,
			"status" => $availableAF['STATUS'],
			"produtos" => array(),
			"dadosEntrega" => $deliveryInfo,
			"dadosCobranca" => $collectionInfo,
			"dadosFaturamento" => $billingInfo,
			"cbc" => $availableAF['CBC']
		);
		
		return $af;
	}
	
	private function getTypeAF($type)
	{
		$typeList = array(
            "N" => "NORMAL",
            "E" => "EXTRA",
            "A" => "ANTECIPACAO",
            "C" => "COMPLEMENTO",
            "I" => "INAUGURACAO",
            "P" => "PLANILHA_ALTERACAO"
        );
        
        return $typeList[$type];
	}
	
	private function getSituationAF($status)
	{
		$statusList = array(
            "1" => "BAIXA_PARCIAL",
            "2" => "BAIXA_TOTAL",
            "3" => "BAIXA_MANUAL",
            "4" => "ENCERRADA",
            "5" => "CANCELADA",
            "7" => "EDI",
            "X" => "EM_ABERTO"
		);
		
		return $statusList[$status];
	}
	
	private function buildDeliveryInfo($availableAF)
	{
		$deliveryInfo = array(
			"unidade" => $availableAF['DADOS_ENTREGA_UNIDADE'],
			"nomeUnidade" => $availableAF['DADOS_ENTREGA_NOMEUNIDADE'],
			"cnpj" => $availableAF['DADOS_ENTREGA_CNPJ'],
			"endereco" => $availableAF['DADOS_ENTREGA_ENDERECO'],
			"bairro" => $availableAF['DADOS_ENTREGA_BAIRRO'],
			"municipio" => $availableAF['DADOS_ENTREGA_MUNICIPIO'],
			"estado" => $availableAF['DADOS_ENTREGA_ESTADO'],
			"cep" => $availableAF['DADOS_ENTREGA_CEP'],
			"horarioEntrega" => $availableAF['DADOS_ENTREGA_HORARIOENTREGA'],
			"contato" => $availableAF['DADOS_ENTREGA_CONTATO'],
			"email" => $availableAF['DADOS_ENTREGA_EMAIL'],
			"telefone" => $availableAF['DADOS_ENTREGA_TELEFONE'],
			"observacao" => $availableAF['DADOS_ENTREGA_OBSERVACAO']
		);
		
		return $deliveryInfo;
	}
	
	private function buildCollectionInfo($availableAF)
	{
		$collectionInfo = array(
			"cnpj" => $availableAF['DADOS_COBRANCA_CNPJ'],
			"endereco" => $availableAF['DADOS_COBRANCA_ENDERECO'],
			"bairro" => $availableAF['DADOS_COBRANCA_BAIRRO'],
			"municipio" => $availableAF['DADOS_COBRANCA_MUNICIPIO'],
			"estado" => $availableAF['DADOS_COBRANCA_ESTADO'],
			"cep" => $availableAF['DADOS_COBRANCA_CEP'],
			"contato" => $availableAF['DADOS_COBRANCA_CONTATO'],
			"email" => $availableAF['DADOS_COBRANCA_EMAIL'],
			"telefone" => $availableAF['DADOS_COBRANCA_TELEFONE']
		);
		
		return $collectionInfo;
	}
	
	private function buildBillingInfo($availableAF)
	{
		$billingInfo = array(
			"cnpj" => $availableAF['DADOS_FATURAMENTO_CNPJ'],
			"endereco" => $availableAF['DADOS_FATURAMENTO_ENDERECO'],
			"bairro" => $availableAF['DADOS_FATURAMENTO_BAIRRO'],
			"municipio" => $availableAF['DADOS_FATURAMENTO_MUNICIPIO'],
			"estado" => $availableAF['DADOS_FATURAMENTO_ESTADO'],
			"cep" => $availableAF['DADOS_FATURAMENTO_CEP']
		);
		
		return $billingInfo;
	}
	
	private function buildProduct($availableAF)
	{
		$product = array(
			"codCliente" => $availableAF['PRODUTO_CODCLIENTE'],
			"nome" => $availableAF['PRODUTO_NOME'],
			"undCliente" => $availableAF['PRODUTO_UNDCLIENTE'],
			"marca" => $availableAF['PRODUTO_MARCA'],
			"dataEntrega" => substr($availableAF['PRODUTO_DATAENTREGA'], 0, 10),
			"quantidade" => $availableAF['PRODUTO_QUANTIDADE'],
			"quantidadePendente" => $availableAF['PRODUTO_QUANTIDADEPENDENTE'],
			"valorUnitario" => $availableAF['PRODUTO_VALORUNITARIO'],
			"valorDesconto" => $availableAF['PRODUTO_VALORDESCONTO'],
			"valorTotal" => $availableAF['PRODUTO_VALORTOTAL']
		);
		
		return $product;
	}
	
	private function buildLogReadAFs($availableAF)
	{
		$af = array(
			"NRSEQDOWNLOADAF" => $this->newCodeSC->getCodeByTableName('LOGDOWNLOADAF', 1, 1, 20),
			"NRSEQENVIOAFORN" => $availableAF['NRSEQENVIOAFORN'],
			"CDFORNECED" => $availableAF['CDFORNECED'],
			"NRAUTOFORN" => $availableAF['NUMERO'],
			"CDFILDEST" => $availableAF['UNIDADEDESTINO'],
			"CDFILLANC" => $availableAF['CDFILLANC'],
			"DSLOGINECO" => "VIA INTEGRACAO"
		);
		
		array_push($this->readAFs, $af);
	}
	
	private function processReadAFs()
	{
		foreach ($this->readAFs as $af) {
			$this->service->setLogDownloadAF($af);
		}
	}
	
}